// Persistence of Vision Ray Tracer Scene Description File
// File: template.pov
// Vers: 3.7
// Desc: Random Space Scene Generator
// Date: January-May 2011
// Auth: Rob Fitzel
// Distances from camera:
// 100-5000 asteroids
// 1000 planets* depends on Extents
// 8000-5000 suns // need to be closer so that they light up planets
// 9000 distant stars
// 20000 nebula shell
// infinity  background {0}     
 
// 
// ini settings to generate 100 space images quickly +KFI0 +KFF100 +KI0 +KF100
// scene constants and overrides ////////////////////////////////////////////// 


#default {finish {ambient 0}} 
// includes ///////////////////////////////////////////////////////////////////
#include "rand.inc" 
#include "transforms.inc"
#include "stones1.inc" // need to override the hi ambient values

// Scene Parameters ///////////////////////////////////////////////////////////////////

// Global Randomizer
// change this value to randomize all parameters 
// to generate a series of random scenes, just add clock to randomizer and vary clock from 0 - some integer value
#declare randomizer = 5060+clock; 


// Distant Stars 

// randomizer
#declare ssd = seed(5113+randomizer); // distant stars 
 
// number of distant stars
#declare min_num_distant_stars = 100;  // 2d: 100
#declare max_num_distant_stars = 2000; // 2d: 3000
 
// scene assumes camera is facing +z, so we don't need to put stars around the sides/ behind camera
// star extents control the horizontal and vertical spread of the stars from +z
#local star_extx = 50; // horizontal spread (40 for stills; 180 for sphere of stars 
#local star_ext = 50; // verical spread in degrees in degrees (can be used to make a milky way of stars

// Suns (big glowing balls)
#declare sun_sd = seed(4209+randomizer); // main star
#declare max_suns = 3;  // not always in view (see SunExtents)
#declare SunExtents = 60;  // maximum angle from +z  set to 50 for stills 180 for full 3d scene 

// Nebula
#declare neb_sd = seed(5428+randomizer); // nebula 

// Lens Dust
#declare dsd = seed(5423+randomizer); // lens dust 
#declare dust_transparency = RRand(0.96,1,dsd);  

// planets
#declare planet_sd = seed(2199+randomizer); // planet
#declare max_num_planets = 5;  // = 1 planet + moons
#declare PlanetMinExtent = <-1000,-500,2000>; // keep planets far (so they dont crash space ship, 
#declare PlanetMaxExtent = < 1000,500,2000>;  // but scale them larger 
#declare planet_ext = 50; // degrees
#declare csd = seed(5235+randomizer); // cracks - doesn't look very nice
#declare max_planet_glow = 5; // atmospheric glow ranges between min and max values
#declare min_planet_glow = 3;
#declare rsd = seed(5432+randomizer); // planetary rings



// Scene Controls /////////////////////////////////////////////////////////////


// camera
#declare view =1; 

// lights
#declare cam_light = 0.2;

// special effects
#declare area   = 1; 

global_settings {  
  assumed_gamma 2.2
  max_trace_level 60 
}



// Camera /////////////////////////////////////////////////////////////////////
#declare CamPos    = vtransform (<0, 0, -100>,transform{rotate y*0});
#declare CamLookAt = <0,0,1000>; 
#declare CamSky    = <0,1,0>;
camera { 

 perspective       // distorts planets when near the edge 
//  ultra_wide_angle   // corrects for most of planet's spherical distortion around edges, but streak lines become very bent
  location  CamPos 
  look_at   CamLookAt 
 // right x*1280/720  
  sky  CamSky 


}  
#declare camera_sky    = CamSky;

//#include "rf_glows.inc" 






// Lights /////////////////////////////////////////////////////////////////////

#if (cam_light>0) 
  light_source {0*x rgb cam_light*<1,0.9,0.9> translate CamPos}
#end 
/*
#if (sun_light>0)
  light_source {0*x rgb sun_light*<1,1,0.9> 
  #if (area)
    area_light
    <50, 0, 0> <0, 0, 50> // lights spread out across this distance (x * z)
    4, 4                // total number of lights in grid (4x*4z = 16 lights)
    adaptive 0          // 0,1,2,3...
    jitter              // adds random softening of light
    circular            // make the shape of the light circular
    orient              // orient light  
  #end       
  translate SunPos}
#end 
*/

// space is black
background {0}

// distant nebula ///////////////////////////////////////////////////////////////////////

#declare t_nebula_max = 46;
#declare t_nebula = array[t_nebula_max]; 

#declare t_nebula[0] = texture {T_Grnt0  finish {phong 0 reflection 0}};
#declare t_nebula[1] = texture {T_Grnt1  finish {phong 0 reflection 0}};
#declare t_nebula[2] = texture {T_Grnt2  finish {phong 0 reflection 0}};
#declare t_nebula[3] = texture {T_Grnt3  finish {phong 0 reflection 0}};
#declare t_nebula[4] = texture {T_Grnt4  finish {phong 0 reflection 0}};
#declare t_nebula[5] = texture {T_Grnt5  finish {phong 0 reflection 0}};
#declare t_nebula[6] = texture {T_Grnt6  finish {phong 0 reflection 0}};
#declare t_nebula[7] = texture {T_Grnt7  finish {phong 0 reflection 0}};
#declare t_nebula[8] = texture {T_Grnt8  finish {phong 0 reflection 0}};
#declare t_nebula[9] = texture {T_Grnt9  finish {phong 0 reflection 0}};
#declare t_nebula[10] = texture {T_Grnt10  finish {phong 0 reflection 0}};
//#declare t_nebula[11] = texture {T_Grnt11  finish {phong 0 reflection 0}}; // blotchy
#declare t_nebula[11] = texture {T_Stone13  finish {phong 0 reflection 0}};
#declare t_nebula[12] = texture {T_Grnt12  finish {phong 0 reflection 0}};
#declare t_nebula[13] = texture {T_Grnt13  finish {phong 0 reflection 0}};
//#declare t_nebula[14] = texture {T_Grnt14  finish {phong 0 reflection 0}};//bad
#declare t_nebula[14] = texture {T_Stone14  finish {phong 0 reflection 0}};
#declare t_nebula[15] = texture {T_Grnt15  finish {phong 0 reflection 0}};
#declare t_nebula[16] = texture {T_Grnt16  finish {phong 0 reflection 0}};
#declare t_nebula[17] = texture {T_Grnt17  finish {phong 0 reflection 0}};
//#declare t_nebula[18] = texture {T_Grnt18  finish {phong 0 reflection 0}};//ugly
#declare t_nebula[18] = texture {T_Stone15  finish {phong 0 reflection 0}};
#declare t_nebula[19] = texture {T_Grnt19  finish {phong 0 reflection 0}};
#declare t_nebula[20] = texture {T_Grnt20  finish {phong 0 reflection 0}};
#declare t_nebula[21] = texture {T_Grnt21  finish {phong 0 reflection 0}};
#declare t_nebula[22] = texture {T_Grnt22  finish {phong 0 reflection 0}};
#declare t_nebula[23] = texture {T_Grnt23  finish {phong 0 reflection 0}};
#declare t_nebula[24] = texture {T_Grnt24  finish {phong 0 reflection 0}};
#declare t_nebula[25] = texture {T_Grnt25  finish {phong 0 reflection 0}};
#declare t_nebula[26] = texture {T_Grnt26  finish {phong 0 reflection 0}};
#declare t_nebula[27] = texture {T_Grnt27  finish {phong 0 reflection 0}};
//#declare t_nebula[28] = texture {T_Grnt28  finish {phong 0 reflection 0}}; //bad
#declare t_nebula[28] = texture {T_Stone16  finish {phong 0 reflection 0}};
#declare t_nebula[29] = texture {T_Grnt29  finish {phong 0 reflection 0}};
#declare t_nebula[30] = texture {T_Stone24  finish {phong 0 reflection 0}};
#declare t_nebula[31] = texture {T_Stone1  finish {phong 0 reflection 0}};
//#declare t_nebula[32] = texture {T_Stone2  finish {phong 0 reflection 0}}; // bad - blotches
#declare t_nebula[32] = texture {T_Stone17  finish {phong 0 reflection 0}};
#declare t_nebula[33] = texture {T_Stone3  finish {phong 0 reflection 0}};
#declare t_nebula[34] = texture {T_Stone4  finish {phong 0 reflection 0}};
#declare t_nebula[35] = texture {T_Stone5  finish {phong 0 reflection 0}};
#declare t_nebula[36] = texture {T_Stone6  finish {phong 0 reflection 0}};
#declare t_nebula[37] = texture {T_Stone7  finish {phong 0 reflection 0}};
#declare t_nebula[38] = texture {T_Stone8  finish {phong 0 reflection 0}};
#declare t_nebula[39] = texture {T_Stone9  finish {phong 0 reflection 0}};
#declare t_nebula[40] = texture {T_Stone10  finish {phong 0 reflection 0}};
#declare t_nebula[41] = texture {T_Stone11  finish {phong 0 reflection 0}};
#declare t_nebula[42] = texture {T_Stone12  finish {phong 0 reflection 0}}; 

//#declare t_nebula[43] = texture {T_Stone13  finish {phong 0 reflection 0}};
//#declare t_nebula[44] = texture {T_Stone14  finish {phong 0 reflection 0}};
//#declare t_nebula[45] = texture {T_Stone15  finish {phong 0 reflection 0}};
//#declare t_nebula[46] = texture {T_Stone16  finish {phong 0 reflection 0}};
//#declare t_nebula[47] = texture {T_Stone17  finish {phong 0 reflection 0}};
//#declare t_nebula[48] = texture {T_Stone18  finish {phong 0 reflection 0}}; //bad
//#declare t_nebula[49] = texture {T_Stone19  finish {phong 0 reflection 0}}; // bad
#declare t_nebula[43] = texture {T_Stone20  finish {phong 0 reflection 0}};
//#declare t_nebula[51] = texture {T_Stone21  finish {phong 0 reflection 0}};// bad
#declare t_nebula[44] = texture {T_Stone22  finish {phong 0 reflection 0}};
#declare t_nebula[45] = texture {T_Stone23  finish {phong 0 reflection 0}};

#declare t_nebula_cloud_max = 23;
#declare t_nebula_cloud = array[t_nebula_cloud_max]; 

#declare t_nebula_cloud[0] = texture {T_Grnt0a  finish {phong 0 reflection 0} scale 5};
#declare t_nebula_cloud[1] = texture {T_Grnt1a  finish {phong 0 reflection 0} scale 5};
#declare t_nebula_cloud[2] = texture {T_Grnt2a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[3] = texture {T_Grnt3a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[4] = texture {T_Grnt4a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[5] = texture {T_Grnt5a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[6] = texture {T_Grnt6a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[7] = texture {T_Grnt7a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[8] = texture {T_Grnt8a  finish {phong 0 reflection 0} scale 5};
//#declare t_nebula_cloud[9] = texture {T_Grnt9a  finish {phong 0 reflection 0}}; // bad 
#declare t_nebula_cloud[9] = texture {T_Grnt23a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[10] = texture {T_Grnt10a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[11] = texture {T_Grnt11a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[12] = texture {T_Grnt12a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[13] = texture {T_Grnt13a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[14] = texture {T_Grnt14a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[15] = texture {T_Grnt15a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[16] = texture {T_Grnt16a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[17] = texture {T_Grnt17a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[18] = texture {T_Grnt18a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[19] = texture {T_Grnt19a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[20] = texture {T_Grnt20a  finish {phong 0 reflection 0}};
#declare t_nebula_cloud[21] = texture {T_Grnt21a  finish {phong 0 reflection 0}};
//#declare t_nebula_cloud[22] = texture {T_Grnt22a  finish {phong 0 reflection 0}}; // bad
#declare t_nebula_cloud[22] = texture {T_Grnt24a  finish {phong 0 reflection 0}};
//#declare t_nebula_cloud[23] = texture {T_Grnt23a  finish {phong 0 reflection 0}};
//#declare t_nebula_cloud[24] = texture {T_Grnt24a  finish {phong 0 reflection 0}};


//#declare t_nebula = texture {	T_Grnt10   scale 10 finish {phong 0 reflection 0}  }
//                   // texture {	T_Grnt1a   scale 100 finish {phong 0 reflection 0}  }; 
#declare selected_nebula_texture = RRand(0,t_nebula_max,neb_sd);

//#declare selected_nebula_texture = clock;
#declare nebula_scaling = RRand(10,50,neb_sd);
#declare t_nebula = texture {	t_nebula[selected_nebula_texture]   scale nebula_scaling   }
                   // texture {t_nebula_cloud [RRand(0,t_nebula_cloud_max,neb_sd)]   scale RRand(1,20,neb_sd) };

#declare selected_dark_nebula_texture = RRand(0,t_nebula_cloud_max,neb_sd);
//#declare selected_dark_nebula_texture = clock;

#declare t_nebula_dark = texture {t_nebula_cloud [selected_dark_nebula_texture]   scale RRand(10,40,neb_sd)*1000};
//#declare t_nebula_dark = texture {t_nebula_cloud [8]   scale RRand(1,4,neb_sd)*100 };

#declare nebula_brighness = RRand(0,0.4,neb_sd);
//#declare nebula_brighness = 0.4;
light_group {
  light_source {0*x rgb nebula_brighness*<0.38,0.702,0.859> translate CamPos}
  //sphere {0,10000 hollow texture {t_nebula scale 2000*<2,1,1>}} 
} 

 
// red highlights
#if (rand(neb_sd) <0.7)
  #declare nebula_highlight_col = CHSV2RGB(<RRand(0,65,neb_sd),0.5,1,0,0>);  
#else
  #declare nebula_highlight_col = CHSV2RGB(<RRand(200,220,neb_sd),0.5,1,0,0>);  
#end 



#declare nebula_highlight_brightness = nebula_highlight_col*RRand(0,3,neb_sd);
light_group {
  light_source {0*x rgb nebula_highlight_brightness  translate CamPos-<1000,5000,0>}
  sphere {0,18500 hollow texture {t_nebula_dark scale RRand(5,20,neb_sd)*2 rotate y*rand(neb_sd)*360} } 
} 
 
light_group {
  light_source {0*x rgb 0  translate CamPos}
  sphere {0,18499 hollow texture {t_nebula_dark scale RRand(1,10,neb_sd)*2 rotate y*RRand(30,270,neb_sd)}} 
}  


// nebula parmeters
#debug "Nebula\n"
#debug "------\n"
#debug concat("nebula texture index = ",      str(selected_nebula_texture,0,0),"\n")
#debug concat("nebula scaling = ", str(nebula_scaling,2,2),"\n")
#debug concat("nebula dark texture index = ", str(selected_dark_nebula_texture,0,0),"\n")
#debug concat("nebula brightness = ", str(nebula_brighness,2,2),"\n")

// distant stars ////////////////////////////////////////////////////////////////////////////////////
#local i=0;

#declare num_distant_stars = RRand(min_num_distant_stars, max_num_distant_stars, ssd);
#while (i<num_distant_stars)
  #if (mod(i,50)=0)
    sphere {0,RRand(2,10,ssd) pigment {rgb <0.3,0.2,0.2>} finish {ambient RRand(0.1,1,ssd)} translate vtransform (<0,0,9000>, transform {rotate <RRand(-star_ext,star_ext,ssd),RRand(-star_extx,star_extx,ssd),0>}) no_shadow}
  #else
    sphere {0,RRand(2,10,ssd) pigment {rgb 0.3} finish {ambient RRand(0.1,1,ssd)} translate vtransform (<0,0,9000>, transform {rotate <RRand(-star_ext,star_ext,ssd),RRand(-star_extx,star_extx,ssd),0>}) no_shadow}
  #end  
  #local i=i+1;  
#end 

#debug "Stars\n"
#debug "-----\n"
#debug concat ("num distant stars = ", str(num_distant_stars,0,0),"\n")

// near stars
#include "colors.inc"  

#if (max_suns > 0)
  #declare num_suns = int(RRand(2,max_suns,sun_sd)); 
  #declare sunPos    = array[num_suns]; 
  #declare sunCol    = array[num_suns]; 
  #declare sunBright = array[num_suns];
  #declare sunScale  = array[num_suns];
  
 // #if (rand(sun_sd) <0.5)
    #local two_streaks = 0;
 // #else
 //   #local two_streaks = 1;
 // #end


  #declare bright_enough_for_lens_dust = 0;
  #local i=0;
  #while (i<num_suns)  

    #debug concat("i=", str(i,2,2), "\n")
    #debug concat("num_suns=", str(num_suns,2,2), "\n")
    
    #if (rand(sun_sd) <0.7)
      #declare sunCol[i] = CHSV2RGB(<RRand(0,65,sun_sd),0.5,1,0,0>);  
    #else
      #declare sunCol[i] = CHSV2RGB(<RRand(200,220,sun_sd),0.5,1,0,0>);  
    #end 
    #declare thisSunExtentX = RRand(-SunExtents,SunExtents,sun_sd);  
    #declare thisSunExtentY = RRand(-SunExtents,SunExtents,sun_sd); 
    //#debug concat("  thisSunExtentX=",
    
    #declare sunPos[i] = vtransform(<0,0,3000>, transform {rotate <thisSunExtentX,thisSunExtentY,0>});
    #declare sunBright[i] = RRand(0.5,1.5,sun_sd);
    #declare sunScale[i] =  RRand(0.5,2.5,sun_sd);
/*
    // compute angle of sun from the view_at
    #declare sunAngle = VAngleD(CamLookAt-CamPos, sunPos[i] - CamPos); 
    #declare extraBright = 1;
    #if (sunAngle < 50)
     // #declare extraBright = 1+max(0,(sunAngle-40))/10;
     // #declare extraBright =  extraBright*extraBright;
      #declare extraBright = 1+exp_peak_val(sunAngle,30, 40, 45);
      #debug concat("sunAngle = ", str(sunAngle,2,2),"\n")
      #debug concat("extraBright = ", str(extraBright,2,2),"\n")
      #declare sunScale[i] =  sunScale[i]*extraBright;
    #end

    #if (sunAngle > 45)
      #ifndef(lens_rainbow_incl)        // just one
        #declare lens_rainbow_incl = 1; 
        #if (rand(sun_sd) >0.5)
          lens_rainbow(1, 2, RRand(1.5,2,sun_sd),   sunPos[i], RRand(0.97,1,sun_sd))  
          #debug "draw rainbow\n"
          #debug concat("lens incl =", str(lens_rainbow_incl,2,2),"\n")
          #debug concat("sun angle =", str(sunAngle,2,2),"\n")
        #end
      #end
    #end
 
    #if (sunBright[i] > 1)
      #if (abs(thisSunExtentX) <30)
        #if (abs(thisSunExtentY) <20)
          #declare bright_enough_for_lens_dust = 1;  // only a central star lights up the dust
        #end
      #end
    #end
    #local i=i+1;
  #end  



  #local i=0;
  #while (i<num_suns) 
                  
    #if (sunBright[i] > 0.9) 
      #if (two_streaks)
        streak(sunBright[i]/3*extraBright, <202,217,234>/255  <1,0.3,1> sunPos[i]+10) 
        streak(sunBright[i]/3*extraBright, <202,217,234>/255 <1,0.3,1> sunPos[i]-10)      
      #else 
        streak(sunBright[i]/3*extraBright, <202,217,234>/255  <1,0.6,1> sunPos[i]+<0,0,10>) 
      #end
    #end  
   
    #if (draw_lens_flares)
      spots(sunBright[i]/4*extraBright, <0.5,0.9,0.7>, 1 sunPos[i])  
    #end
   circ_glow_behind(sunBright[i], sunCol[i], sunScale[i], sunPos[i])  
*/ 
  
    light_source {0*x rgb (sunCol[i]*sunBright[i]) translate sunPos[i]
      #if (area)
        area_light
        <50, 0, 0> <0, 0, 50> // lights spread out across this distance (x * z)
        4, 4                // total number of lights in grid (4x*4z = 16 lights)
        adaptive 0          // 0,1,2,3...
        jitter              // adds random softening of light
        circular            // make the shape of the light circular
        orient              // orient light  
      #end       


    } 
 
  
    #local i=i+1;
  #end 

#debug "Suns\n"
#debug "-----\n"
#debug concat ("num suns = ", str(num_suns,0,0),"\n")
#debug concat ("bright enough for visible lens dust", str(bright_enough_for_lens_dust,0,0),"\n")
#end // num stars


/*
streak(RRand(0,0.5,sun_sd), <0.5,0.5,0.5> <5,2,1> Sun1Pos)
streak(RRand(0,0.5,sun_sd), <0.5,0.5,0.5> <5,2,1> Sun2Pos)
streak(RRand(0,0.5,sun_sd), <0.5,0.5,0.5> <5,2,1> Sun3Pos)  
#if (draw_lens_flares)
  spots(0.5, <0.5,0.9,0.7>, 1 Sun1Pos)  
  spots(0.5, <0.5,0.9,0.7>, 1 Sun2Pos)  
  spots(0.5, <0.5,0.9,0.7>, 1 Sun3Pos)  
#end
circ_glow_behind(RRand(0.1,1.5,sun_sd), sun1Col, RRand(0.4,1.3,sun_sd), Sun1Pos)  
circ_glow_behind(RRand(0.1,1.5,sun_sd), sun2Col, RRand(0.4,1.3,sun_sd), Sun2Pos)  
circ_glow_behind(RRand(0.1,1.5,sun_sd), sun3Col, RRand(0.4,1.3,sun_sd), Sun3Pos)  
*/
// lens dust controlled by random dust 
/*
#if (bright_enough_for_lens_dust)
  #declare dust_brightness = 1*extraBright;
  #declare num_motes = 100;
  #include "rf_lensdust.inc"
#end
*/
   

//planets 
#declare num_planets = int(RRand(1,max_num_planets,planet_sd));
#if (num_planets > 0)
    #declare planet = array[num_planets];
    
    #declare t_planet_max = 25;  // reset to 24 as last one is a test
    #declare t_planet = array[t_planet_max]; 
    
    #declare t_planet[0] = texture {T_Stone24 finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[1] = texture {T_Stone1  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[2] = texture {T_Stone2  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[3] = texture {T_Stone3  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[4] = texture {T_Stone4  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[5] = texture {T_Stone5  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[6] = texture {T_Stone6  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[7] = texture {T_Stone7  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[8] = texture {T_Stone8  finish {phong 0 ambient 0 reflection 0}};    //
    #declare t_planet[9] = texture {T_Stone9  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[10] = texture {T_Stone10  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[11] = texture {T_Stone11  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[12] = texture {T_Stone12  finish {phong 0 ambient 0 reflection 0}};  //
    #declare t_planet[13] = texture {T_Stone13  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[14] = texture {T_Stone14  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[15] = texture {T_Stone15  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[16] = texture {T_Stone16  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[17] = texture {T_Stone17  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[18] = texture {T_Stone18  finish {diffuse 0.6 phong 0 ambient 0 reflection 0}};  // too bright
    #declare t_planet[19] = texture {T_Stone19  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[20] = texture {T_Stone20  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[21] = texture {T_Stone21  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[22] = texture {T_Stone22  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[23] = texture {T_Stone23  finish {phong 0 ambient 0 reflection 0}};
    #declare t_planet[24] = texture {pigment {rgb 0.9}};



    
#declare p_ring_base = rgbt <0.5,0.5,0.5,0.3>;
#declare p_ring = 
pigment { 
 cylindrical
 color_map {
   [0.35  rgbt p_ring_base+<0,0,0,rand(rsd)>]
   [0.38  rgbt p_ring_base+<0,0,0,rand(rsd)>]
   [0.39  rgbt p_ring_base+<0,0,0,rand(rsd)>]
   [0.4  rgbt p_ring_base+<0,0,0,rand(rsd)>]
   [0.45  rgbt p_ring_base+<0,0,0,rand(rsd)>]
   [0.55  rgbt p_ring_base+<0,0,0,rand(rsd)>]
  }
}; 
    
    
    
      //#declare t_planet = texture {pigment {granite color_map {[0 rgb 0*<0.337,0.459,0.561>][1 rgb 10*<0.325,0.569,0.506>]} scale 0.1} };  
     // #declare t_planet = texture { t_planet[RRand(0,t_planet_max,planet_sd)]  scale RRand(1,8,planet_sd) finish {phong 0 reflection 0 brilliance 10}  };
    
    //#declare planet = array[num_planets];
    
    #local i=0; 
    #while (i<num_planets)


      
      #declare t_planet_atmosphere = 
            texture {pigment {rgbt <RRand(0.35,0.55,planet_sd),RRand(0.35,0.55,planet_sd),RRand(0.35,0.55,planet_sd),RRand(0.7,1,planet_sd)>} 
                    finish {ambient 0 phong 0.1 phong_size 1}
                    };
      #declare p_planet_atmosphere2 = pigment {rgbt <RRand(0.35,0.55,planet_sd),RRand(0.35,0.55,planet_sd),RRand(0.35,0.55,planet_sd),RRand(0.7,1,planet_sd)>}; 
      #declare p_planet_atmosphere1 = pigment {rgb RRand(min_planet_glow, max_planet_glow,planet_sd)};

      
      #declare p_planet_atmosphere = 
      pigment {
        aoi
        pigment_map {
        [0.5 p_planet_atmosphere1]
        [0.7 p_planet_atmosphere2]
        }
      };

      #declare t_planet_atmosphere = 
            texture {pigment {p_planet_atmosphere}
                    finish {ambient 0}  
            } 


#macro crack_me_up()
  #local t_magma  = texture {pigment {bozo color_map {[0 rgb <1,1,0>*RRand(1,2,csd)][0.2 rgb <0.855,0.278,0.106>*RRand(1,2,csd)][1 rgb <0.573,0.047,0.075>*RRand(1,2,csd)]  } warp {turbulence 2 } } scale 4 }; 
  #local t_crack2 = texture {gradient z texture_map {[0 t_magma][1 t_planet_surface]} scale 0.5};
  #local crack_width = RRand(0.005,0.008,csd);
  #declare t_planet_surface = 
  texture {
        crackle
        metric 2.4
        offset 1.0
        texture_map {
          [crack_width t_crack2]
          [crack_width t_planet_surface scale 1/3]
        }   
       warp {turbulence RRand(0.2,0.6,csd) }
       scale RRand(0.2,0.5,csd)*6 
       rotate rand(csd)*360
  }; 



#end   

#declare t_planet_surface = texture {t_planet[RRand(0,t_planet_max,planet_sd)] scale RRand(0.3,8,planet_sd) rotate y*rand(planet_sd)*360};
#local planet_cracks = 0;
/*
#if (rand(csd) < 0.05) // cracks are rare
  crack_me_up() // add planet wide tectonic cracks to planet surface
  #local planet_cracks = 1;
#end
*/    
      #declare planet = 
      union { 
        sphere {0,3   texture {t_planet_surface} 
                      #if (planet_cracks = 0) normal {granite bump_size 100*rand(planet_sd)} #end
               }  
        #if (rand(planet_sd) < 0.7 )      
          sphere {0,3.0001 texture {t_planet_atmosphere}}  
        #end

        #if (rand(planet_sd) < 0.02)
        disc {0 y 8 RRand(5,7,rsd)  
            texture { 
              pigment {onion 
                  color_map {  

                    [0   rgbt <1,1,1,0.25>]
                    #local k=0;
                    #while (k<1) 
                      #local g = RRand(0.001,0.025,rsd);  // gap 
                      #local w =  RRand(0.005,0.05,rsd);  // width
                      [k+g+w   rgbt <1,1,1,0.25>]
                      [k+g+w   rgbt 1]
                      [k+g+w+w rgbt 1]
                      [k+g+w+w rgbt <1,1,1,0.25>]
                             
                      #local k=k+g+w;
                    #end   
                    [1 rgbt <1,1,1,0.25>] 
                  } 
                 scale 2
                 } 

                
              }
 
              rotate z*rand(planet_sd)*90 rotate y*rand(planet_sd)*90}

        #end
        };
        #if (i=0)
          #declare planet_size = RRand(1,400,planet_sd); // first planet is big 
          //#declare planet_pos = <RRand(PlanetMinExtent.x,PlanetMaxExtent.x,planet_sd),RRand(PlanetMinExtent.y,PlanetMaxExtent.y,planet_sd),RRand(PlanetMinExtent.z,PlanetMaxExtent.z,planet_sd)>;
          #declare planet_pos = vtransform(<0,0,2000>, transform {rotate x*RRand(-planet_ext,planet_ext,planet_sd) rotate y*RRand(-planet_ext,planet_ext,planet_sd)});
          object {planet scale planet_size translate planet_pos}
        #else // next ones are really moons
          #declare moon_size = planet_size*RRand(0.2,0.01,planet_sd); 
          #declare moon_pos = vtransform(<0,0,0>, transform{translate x*(planet_size+moon_size+planet_size*RRand(2,5,planet_sd)) rotate y*rand(planet_sd)*360 rotate x*RRand(-90,90,planet_sd) translate planet_pos});
          object {planet scale moon_size translate moon_pos }
        #end
   
    
      #local i=i+1;
    #end  

#debug "Planets\n"
#debug "-------\n"
#debug concat("num planets =", str(num_planets,0,0), "\n")

#end // num planets > 0







